@@ -11,15 +11,10 @@ isPLNPCAfamily     <- function(Robject) {inherits(Robject, "PLNPCAfamily"    )}
 #'
 #' @name plot.PLNPCAfamily
 #'
-#' @param x an R6 object with class [`PLNfamily`]
-#' @param criteria vector of characters. The criteria to plot in c("loglik", "BIC", "ICL", "R_squared").
-#' Default is  c("loglik", "BIC", "ICL").
-#' @param annotate logical: should the value of approximated R squared be added to the plot?
-#' @param ... additional parameters for S3 compatibility. Not used
+#' @param x an R6 object with class [`PLNPCAfamily`]
+#' @inheritParams plot.PLNfamily
+#' @inherit plot.PLNfamily return details
 #'
-#' @return Produces a plot  representing the evolution of the criteria of the different models considered,
-#' highlighting the best model in terms of BIC and ICL (the greater, the better).
-#' These criteria have the form 'loglik - 1/2 * penalty' so that they are on the same scale as the model loglikelihood.
 #' @examples
 #' data(trichoptera)
 #' trichoptera <- prepare_data(trichoptera$Abundance, trichoptera$Covariate)
@@ -28,9 +23,9 @@ isPLNPCAfamily     <- function(Robject) {inherits(Robject, "PLNPCAfamily"    )}
 #' plot(myPCAs)
 #' }
 #' @export
-plot.PLNPCAfamily <- function(x, criteria = c("loglik", "BIC", "ICL"), annotate = TRUE, ...) {
+plot.PLNPCAfamily <- function(x, criteria = c("loglik", "BIC", "ICL"), reverse = FALSE, ...) {
   stopifnot(isPLNfamily(x))
-  x$plot(criteria, annotate)
+  x$plot(criteria, reverse)
 }
 
 #' @describeIn getModel Model extraction for [`PLNPCAfamily`]
@@ -42,7 +37,7 @@ getModel.PLNPCAfamily <- function(Robject, var, index = NULL) {
 
 #' @describeIn getBestModel Model extraction for [`PLNPCAfamily`]
 #' @export
-getBestModel.PLNPCAfamily <- function(Robject, crit = c("ICL", "BIC", "R_squared"), ...) {
+getBestModel.PLNPCAfamily <- function(Robject, crit = c("ICL", "BIC"), ...) {
   stopifnot(isPLNPCAfamily(Robject))
   Robject$getBestModel(match.arg(crit))
 }
