@@ -22,16 +22,15 @@ test_that("PLNLDA fit: check classes, getters and field access",  {
   expect_null(coef(model))
   expect_lt(sum((model$group_means - model$model_par$Theta)^2), .Machine$double.eps)
   expect_equal(sigma(model), model$model_par$Sigma)
-  expect_equal(vcov(model), model$fisher$mat)
+  expect_equal(vcov(model), model$vcov_coef)
 
   expect_true(inherits(model$group_means, "matrix"))
   expect_true(inherits(sigma(model), "matrix"))
   # expect_true(inherits(vcov(model), "dsCMatrix"))
-
-  expect_equal(dim(vcov(model)), c(model$d * model$p, model$d * model$p))
+  # expect_equal(dim(vcov(model)), c(model$d * model$p, model$d * model$p))
 
   ## fields and active bindings
-  expect_equal(dim(model$model_par$B), c(p, p))
+  expect_equal(dim(model$model_par$C), c(p, p))
   expect_equal(dim(model$model_par$Sigma), c(p, p))
   expect_equal(dim(model$var_par$M), c(n, p))
   expect_equal(dim(model$var_par$S), c(n, p))
@@ -48,7 +47,7 @@ test_that("PLNLDA fit: check classes, getters and field access",  {
   ## S3 methods
   expect_equal(dim(fitted(model)), c(n, p))
   expect_equal(sigma(model), model$model_par$Sigma)
-  expect_equal(vcov(model, "main"), model$fisher$mat)
+  expect_equal(vcov(model, "main"), model$vcov_coef)
   expect_equal(vcov(model, "covariance"), model$model_par$Sigma)
   expect_equal(vcov(model, "covariance"), sigma(model))
 
