@@ -4,7 +4,7 @@ data(trichoptera)
 trichoptera <- prepare_data(trichoptera$Abundance, trichoptera$Covariate)
 models <- PLNPCA(Abundance ~ 1, data = trichoptera)
 X <- model.matrix(Abundance ~ 1, data = trichoptera)
-myPLNfit <- getBestModel(models)
+myPLNfit <- getModel(models, 3)
 
 test_that("PLNPCA fit: check classes, getters and field access", {
 
@@ -16,7 +16,7 @@ test_that("PLNPCA fit: check classes, getters and field access", {
   ## fields and active bindings
   expect_equal(dim(myPLNfit$latent), dim(Y))
   expect_equal(dim(myPLNfit$model_par$Theta), c(ncol(Y), ncol(X)))
-  expect_equal(dim(myPLNfit$model_par$B), c(ncol(Y), myPLNfit$rank))
+  expect_equal(dim(myPLNfit$model_par$C), c(ncol(Y), myPLNfit$rank))
   expect_equal(dim(myPLNfit$model_par$Sigma), c(ncol(Y), ncol(Y)))
   expect_equal(dim(myPLNfit$var_par$M), c(nrow(Y), myPLNfit$rank))
   expect_equal(dim(myPLNfit$var_par$S), c(nrow(Y), myPLNfit$rank))
@@ -44,13 +44,15 @@ test_that("PLNPCA fit: check classes, getters and field access", {
   expect_equal(length(myPLNfit$ind$dist), n)
 
   ## S3 methods
-  expect_equal(coefficients(myPLNfit), myPLNfit$model_par$Theta)
+  expect_equal(coefficients(myPLNfit), myPLNfit$model_par$B)
   expect_equal(dim(fitted(myPLNfit)), dim(Y))
   expect_equal(sigma(myPLNfit), myPLNfit$model_par$Sigma)
-  expect_equal(vcov(myPLNfit, "main"), myPLNfit$fisher$mat)
+  expect_equal(vcov(myPLNfit, "main"), myPLNfit$vcov_coef)
   expect_equal(vcov(myPLNfit, "covariance"), myPLNfit$model_par$Sigma)
   expect_equal(vcov(myPLNfit, "covariance"), sigma(myPLNfit))
-  # expect_equal(dim(standard_error(myPLNfit)), dim(coefficients(myPLNfit)))
+  expect_warning(sd_plnfit <- standard_error(myPLNfit), "Standard error of B is not implemented yet for PLNPCA models")
+  expect_equal(dim(sd_plnfit), dim(coefficients(myPLNfit)))
+  expect_error(standard_error(myPLNfit, parameter = "Omega"), "Omega is not estimated as such in PLNPCA models")
 
   expect_true(inherits(plot(myPLNfit, map = "variable", plot = FALSE), "ggplot"))
   expect_true(inherits(plot(myPLNfit, map = "individual", plot = FALSE), "ggplot"))
@@ -62,10 +64,10 @@ test_that("PLNPCA fit: check classes, getters and field access", {
   expect_true(inherits(myPLNfit$plot_PCA(plot = FALSE), "grob"))
 
   ## R6 methods: VEstep
-  ve_results <- myPLNfit$VEstep(covariates = X, offsets = O, responses = Y)
+  ve_results <- myPLNfit$optimize_vestep(covariates = X, offsets = O, responses = Y)
   expect_equal(dim(ve_results$M), c(n, myPLNfit$rank))
-  expect_equal(dim(ve_results$S2), c(n, myPLNfit$rank))
-  expect_length(ve_results$log.lik, n)
+  expect_equal(dim(ve_results$S), c(n, myPLNfit$rank))
+  expect_length(ve_results$Ji, n)
   expect_equal(ve_results$M, unname(myPLNfit$var_par$M), tolerance = 1e-1)
   ## R6 methods: project()
   scores <- myPLNfit$project(newdata = trichoptera)
@@ -82,7 +84,7 @@ test_that("Bindings for factoextra return sensible values", {
   expect_gte(min(myPLNfit$eig[, "eigenvalue"]), 0)
   expect_gte(min(myPLNfit$eig[, "percentage of variance"]), 0)
   expect_lte(max(myPLNfit$eig[, "percentage of variance"]), 100 * myPLNfit$R_squared)
-  expect_equivalent(tail(myPLNfit$eig[, "cumulative percentage of variance"], n = 1), 100 * myPLNfit$R_squared)
+  expect_equal(tail(myPLNfit$eig[, "cumulative percentage of variance"], n = 1), 100 * myPLNfit$R_squared, check.attributes = FALSE)
   ## $var
   .var <- myPLNfit$var
   cor_range <- range(.var$cor)
@@ -91,22 +93,21 @@ test_that("Bindings for factoextra return sensible values", {
   cos2_range <- range(.var$cos2)
   expect_gte(cos2_range[1], 0)
   expect_lte(cos2_range[2], 1)
-  expect_equivalent(rowSums(.var$cos2), rep(1, myPLNfit$p))
-  expect_equivalent(colSums(.var$contrib), rep(100, myPLNfit$rank))
+  expect_equal(rowSums(.var$cos2), rep(1, myPLNfit$p), check.attributes = FALSE)
+  expect_equal(colSums(.var$contrib), rep(100, myPLNfit$rank), check.attributes = FALSE)
   ## $ind
   .ind <- myPLNfit$ind
   cos2_range <- range(.ind$cos2)
   expect_gte(cos2_range[1], 0)
   expect_lte(cos2_range[2], 1)
-  expect_equivalent(rowSums(.ind$cos2), rep(1, myPLNfit$n))
-  expect_equivalent(colSums(.ind$contrib), rep(100, myPLNfit$rank))
-  expect_equivalent(colSums(.ind$coord), rep(0, myPLNfit$rank))
+  expect_equal(rowSums(.ind$cos2), rep(1, myPLNfit$n), check.attributes = FALSE)
+  expect_equal(colSums(.ind$contrib), rep(100, myPLNfit$rank), check.attributes = FALSE)
+  expect_equal(colSums(.ind$coord), rep(0, myPLNfit$rank), check.attributes = FALSE)
 })
 
 
 test_that("Louis-type Fisher matrices are not available for PLNPCA", {
-  expect_error(myPLNfit$compute_fisher(type = "louis", X = X),
-               "Louis approximation scheme not available yet for object of class PLNPLCA, use type = \"wald\" instead.")
+  expect_error(myPLNfit$compute_fisher(type = "louis", X = X))
 })
 
 test_that("plot_PCA works one axis only:", {
@@ -121,9 +122,8 @@ test_that("plot_PCA works for 4 or more axes:", {
 })
 
 test_that("PLNPCA fit: check print message",  {
-
   output <- paste(
-"Poisson Lognormal with rank constrained for PCA (rank = 4)
+"Poisson Lognormal with rank constrained for PCA (rank = 3)
 ==================================================================",
 capture_output(print(as.data.frame(round(myPLNfit$criteria, digits = 3), row.names = ""))),
 "==================================================================
